@@ -1,9 +1,14 @@
-/*
- * FlowData.h
- */
+/*******************************************************************************
+ Infomap software package for multi-level network clustering
+ Copyright (c) 2013, 2014 Daniel Edler, Anton Eriksson, Martin Rosvall
 
-#ifndef SRC_CLUSTERING_FLOWDATA_H_
-#define SRC_CLUSTERING_FLOWDATA_H_
+ This file is part of the Infomap software package.
+ See file LICENSE_AGPLv3.txt for full license details.
+ For more information, see <http://www.mapequation.org>
+ ******************************************************************************/
+
+#ifndef FLOWDATA_H_
+#define FLOWDATA_H_
 
 #include <ostream>
 #include <utility>
@@ -11,43 +16,16 @@
 namespace infomap {
 
 struct FlowData {
-  FlowData(double flow = 0.0)
-      : flow(flow),
-        enterFlow(0.0),
-        exitFlow(0.0),
-        teleportFlow(0.0),
-        teleportSourceFlow(0.0),
-        teleportWeight(0.0),
-        danglingFlow(0.0) {}
+  double flow = 0.0;
+  double enterFlow = 0.0;
+  double exitFlow = 0.0;
+  double teleportFlow = 0.0;
+  double teleportSourceFlow = 0.0;
+  double teleportWeight = 0.0;
+  double danglingFlow = 0.0;
 
-  FlowData(const FlowData& other)
-      : flow(other.flow),
-        enterFlow(other.enterFlow),
-        exitFlow(other.exitFlow),
-        teleportFlow(other.teleportFlow),
-        teleportSourceFlow(other.teleportSourceFlow),
-        teleportWeight(other.teleportWeight),
-        danglingFlow(other.danglingFlow) {}
-
-  FlowData& operator=(const FlowData& other)
-  {
-    flow = other.flow;
-    enterFlow = other.enterFlow;
-    exitFlow = other.exitFlow;
-    teleportFlow = other.teleportFlow;
-    teleportSourceFlow = other.teleportSourceFlow;
-    teleportWeight = other.teleportWeight;
-    danglingFlow = other.danglingFlow;
-    return *this;
-  }
-
-  double flow;
-  double enterFlow;
-  double exitFlow;
-  double teleportFlow;
-  double teleportSourceFlow;
-  double teleportWeight;
-  double danglingFlow;
+  FlowData() = default;
+  FlowData(double flow) : flow(flow) { }
 
   FlowData& operator+=(const FlowData& other)
   {
@@ -75,34 +53,29 @@ struct FlowData {
 
   friend std::ostream& operator<<(std::ostream& out, const FlowData& data)
   {
-    // return out << "flow: " << data.flow << ", enter: " << data.enterFlow << ", exit: " << data.exitFlow;
-    return out << "flow: " << data.flow << ", enter: " << data.enterFlow <<
-				", exit: " << data.exitFlow << ", teleWeight: " << data.teleportWeight <<
-				", danglingFlow: " << data.danglingFlow << ", teleFlow: " << data.teleportFlow;
+    return out << "flow: " << data.flow << ", enter: " << data.enterFlow << ", exit: " << data.exitFlow
+               << ", teleWeight: " << data.teleportWeight << ", danglingFlow: " << data.danglingFlow
+               << ", teleFlow: " << data.teleportFlow;
   }
 };
 
-
 struct DeltaFlow {
   unsigned int module = 0;
   double deltaExit = 0.0;
   double deltaEnter = 0.0;
   unsigned int count = 0;
 
-  virtual ~DeltaFlow() = default;
-
-  DeltaFlow() = default;
-
   explicit DeltaFlow(unsigned int module, double deltaExit, double deltaEnter)
       : module(module),
         deltaExit(deltaExit),
-        deltaEnter(deltaEnter),
-        count(0) {}
+        deltaEnter(deltaEnter) { }
 
+  DeltaFlow() = default;
   DeltaFlow(const DeltaFlow&) = default;
-  DeltaFlow& operator=(const DeltaFlow&) = default;
   DeltaFlow(DeltaFlow&&) = default;
+  DeltaFlow& operator=(const DeltaFlow&) = default;
   DeltaFlow& operator=(DeltaFlow&&) = default;
+  virtual ~DeltaFlow() = default;
 
   DeltaFlow& operator+=(const DeltaFlow& other)
   {
@@ -113,7 +86,7 @@ struct DeltaFlow {
     return *this;
   }
 
-  void reset()
+  virtual void reset()
   {
     module = 0;
     deltaExit = 0.0;
@@ -121,7 +94,7 @@ struct DeltaFlow {
     count = 0;
   }
 
-  friend void swap(DeltaFlow& first, DeltaFlow& second)
+  friend void swap(DeltaFlow& first, DeltaFlow& second) noexcept
   {
     std::swap(first.module, second.module);
     std::swap(first.deltaExit, second.deltaExit);
@@ -139,12 +112,12 @@ struct MemDeltaFlow : DeltaFlow {
   double sumDeltaPlogpPhysFlow = 0.0;
   double sumPlogpPhysFlow = 0.0;
 
-  MemDeltaFlow() : DeltaFlow() {}
+  MemDeltaFlow() = default;
 
   explicit MemDeltaFlow(unsigned int module, double deltaExit, double deltaEnter, double sumDeltaPlogpPhysFlow = 0.0, double sumPlogpPhysFlow = 0.0)
       : DeltaFlow(module, deltaExit, deltaEnter),
         sumDeltaPlogpPhysFlow(sumDeltaPlogpPhysFlow),
-        sumPlogpPhysFlow(sumPlogpPhysFlow) {}
+        sumPlogpPhysFlow(sumPlogpPhysFlow) { }
 
   MemDeltaFlow& operator+=(const MemDeltaFlow& other)
   {
@@ -154,14 +127,14 @@ struct MemDeltaFlow : DeltaFlow {
     return *this;
   }
 
-  void reset()
+  void reset() override
   {
     DeltaFlow::reset();
     sumDeltaPlogpPhysFlow = 0.0;
     sumPlogpPhysFlow = 0.0;
   }
 
-  friend void swap(MemDeltaFlow& first, MemDeltaFlow& second)
+  friend void swap(MemDeltaFlow& first, MemDeltaFlow& second) noexcept
   {
     swap(static_cast<DeltaFlow&>(first), static_cast<DeltaFlow&>(second));
     std::swap(first.sumDeltaPlogpPhysFlow, second.sumDeltaPlogpPhysFlow);
@@ -174,24 +147,19 @@ struct MemDeltaFlow : DeltaFlow {
   }
 };
 
-
 struct PhysData {
-  explicit PhysData(unsigned int physNodeIndex, double sumFlowFromM2Node = 0.0)
-      : physNodeIndex(physNodeIndex), sumFlowFromM2Node(sumFlowFromM2Node) {}
-
-  PhysData(const PhysData& other) = default;
-
   unsigned int physNodeIndex;
   double sumFlowFromM2Node; // The amount of flow from the memory node in this physical node
 
+  explicit PhysData(unsigned int physNodeIndex, double sumFlowFromM2Node = 0.0)
+      : physNodeIndex(physNodeIndex), sumFlowFromM2Node(sumFlowFromM2Node) { }
+
   friend std::ostream& operator<<(std::ostream& out, const PhysData& data)
   {
     return out << "physNodeIndex: " << data.physNodeIndex << ", sumFlowFromM2Node: " << data.sumFlowFromM2Node;
   }
 };
 
-
 } // namespace infomap
 
-
-#endif /* SRC_CLUSTERING_FLOWDATA_H_ */
+#endif // FLOWDATA_H_
