@@ -13,7 +13,6 @@
 #include "FlowData.h"
 #include "InfoEdge.h"
 #include "iterators/infomapIterators.h"
-#include "iterators/treeIterators.h"
 #include "iterators/IterWrapper.h"
 #include "../utils/MetaCollection.h"
 
@@ -170,6 +169,8 @@ public:
   // ---------------------------- Infomap ----------------------------
   InfomapBase& getInfomap();
 
+  const InfomapBase& getInfomap() const;
+
   InfomapBase& setInfomap(InfomapBase*);
 
   InfoNode* getInfomapRoot() noexcept;
