@@ -40,13 +40,11 @@ namespace detail {
 class InfomapBase : public InfomapConfig<InfomapBase> {
   template <typename Objective>
   friend class InfomapOptimizer;
-  friend class Output;
 
   void initOptimizer(bool forceNoMemory = false);
 
 public:
   using PartitionQueue = detail::PartitionQueue;
-  using PerLevelStat = detail::PerLevelStat;
 
   InfomapBase() : InfomapConfig<InfomapBase>() { initOptimizer(); }
 
@@ -62,15 +60,9 @@ public:
   virtual ~InfomapBase() = default;
 
   // ===================================================
-  // Getters
+  // Iterators
   // ===================================================
 
-  Network& network() { return m_network; }
-  const Network& network() const { return m_network; }
-
-  InfoNode& root() { return m_root; }
-  const InfoNode& root() const { return m_root; }
-
   InfomapIterator iterTree(int maxClusterLevel = 1) { return { &root(), maxClusterLevel }; }
 
   InfomapIteratorPhysical iterTreePhysical(int maxClusterLevel = 1) { return { &root(), maxClusterLevel }; }
@@ -85,7 +77,17 @@ public:
 
   InfomapIterator begin(int maxClusterLevel = 1) { return { &root(), maxClusterLevel }; }
 
-  InfomapIterator end() { return InfomapIterator(nullptr); }
+  InfomapIterator end() const { return InfomapIterator(nullptr); }
+
+  // ===================================================
+  // Getters
+  // ===================================================
+
+  Network& network() { return m_network; }
+  const Network& network() const { return m_network; }
+
+  InfoNode& root() { return m_root; }
+  const InfoNode& root() const { return m_root; }
 
   unsigned int numLeafNodes() const { return m_leafNodes.size(); }
 
@@ -133,6 +135,38 @@ public:
     return oneLevelCodelength < 1e-16 ? 0 : 1.0 - codelength() / oneLevelCodelength;
   }
 
+  const Date& getStartDate() const { return m_startDate; }
+  const Stopwatch& getElapsedTime() const { return m_elapsedTime; }
+
+  std::vector<InfoNode*>& activeNetwork() const { return *m_activeNetwork; }
+
+  std::map<unsigned int, std::vector<unsigned int>> getMultilevelModules(bool states = false);
+
+  // ===================================================
+  // IO
+  // ===================================================
+
+  std::ostream& toString(std::ostream& out) const { return m_optimizer->toString(out); }
+
+  // ===================================================
+  // Run
+  // ===================================================
+
+  using InitialPartition = std::map<unsigned int, unsigned int>;
+
+  const InitialPartition& getInitialPartition() const { return m_initialPartition; }
+
+  InfomapBase& setInitialPartition(const InitialPartition& moduleIds)
+  {
+    m_initialPartition = moduleIds;
+    return *this;
+  }
+
+  void run(const std::string& parameters = "");
+
+  void run(Network& network);
+
+private:
   bool isFullNetwork() const { return m_isMain && m_aggregationLevel == 0; }
   bool isFirstLoop() const { return m_tuneIterationIndex == 0 && isFullNetwork(); }
 
@@ -176,41 +210,15 @@ public:
   }
 
   bool isTopLevel() const { return (m_subLevel & (SUPER_LEVEL_ADDITION - 1)) == 0; }
+
   bool isSuperLevelOnTopLevel() const { return m_subLevel == SUPER_LEVEL_ADDITION; }
+
   bool isMainInfomap() const { return m_isMain; }
 
   bool haveHardPartition() const { return !m_originalLeafNodes.empty(); }
 
-  std::vector<InfoNode*>& activeNetwork() const { return *m_activeNetwork; }
-
-  // ===================================================
-  // IO
-  // ===================================================
-
-  std::ostream& toString(std::ostream& out) const { return m_optimizer->toString(out); }
-
-  const std::map<unsigned int, unsigned int>& getInitialPartition() const { return m_initialPartition; }
-
-  // ===================================================
-  // Init
-  // ===================================================
-
   double calcEntropyRate();
 
-  InfomapBase& setInitialPartition(const std::map<unsigned int, unsigned int>& moduleIds)
-  {
-    m_initialPartition = moduleIds;
-    return *this;
-  }
-
-  // ===================================================
-  // Run
-  // ===================================================
-
-  void run(const std::string& parameters = "");
-
-  void run(Network& network);
-
   // ===================================================
   // Run: *
   // ===================================================
@@ -289,9 +297,7 @@ public:
 
   void restoreHardPartition();
 
-  void sortTreeOnFlow() { root().sortChildrenOnFlow(); }
-
-  void writeResult();
+  void writeResult(int trial = -1);
 
   // ===================================================
   // runPartition: *
@@ -326,7 +332,7 @@ public:
    * @param includeRoot Also calculate the codelength on the root node
    * @return the hierarchical codelength
    */
-  double calcCodelengthOnTree(bool includeRoot = true);
+  double calcCodelengthOnTree(InfoNode& root, bool includeRoot = true) const;
 
   // ===================================================
   // Run: Partition: *
@@ -359,9 +365,9 @@ public:
     return m_optimizer->consolidateModules(replaceExistingModules);
   }
 
-  void calculateNumNonTrivialTopModules();
+  unsigned int calculateNumNonTrivialTopModules() const;
 
-  unsigned int calculateMaxDepth();
+  unsigned int calculateMaxDepth() const;
 
   // ===================================================
   // Partition: findTopModulesRepeatedly: *
@@ -408,8 +414,9 @@ public:
 
   void queueLeafModules(PartitionQueue& partitionQueue);
 
-  bool processPartitionQueue(PartitionQueue& queue, PartitionQueue& nextLevel);
+  bool processPartitionQueue(PartitionQueue& queue, PartitionQueue& nextLevel) const;
 
+public:
   // ===================================================
   // Output: *
   // ===================================================
@@ -421,7 +428,7 @@ public:
    * @param states if memory network, print the state-level network without merging physical nodes within modules
    * @return the filename written to
    */
-  std::string writeTree(const std::string& filename = "", bool states = false) { return Output::writeTree(*this, filename, states); }
+  std::string writeTree(const std::string& filename = "", bool states = false) { return infomap::writeTree(*this, m_network, filename, states); }
 
   /**
    * Write flow tree to a .ftree file.
@@ -432,7 +439,7 @@ public:
    * @param states if memory network, print the state-level network without merging physical nodes within modules
    * @return the filename written to
    */
-  std::string writeFlowTree(const std::string& filename = "", bool states = false) { return Output::writeFlowTree(*this, filename, states); }
+  std::string writeFlowTree(const std::string& filename = "", bool states = false) { return infomap::writeFlowTree(*this, m_network, filename, states); }
 
   /**
    * Write Newick tree to a .tre file.
@@ -441,11 +448,11 @@ public:
    * @param states if memory network, print the state-level network without merging physical nodes within modules
    * @return the filename written to
    */
-  std::string writeNewickTree(const std::string& filename = "", bool states = false) { return Output::writeNewickTree(*this, filename, states); }
+  std::string writeNewickTree(const std::string& filename = "", bool states = false) { return infomap::writeNewickTree(*this, filename, states); }
 
-  std::string writeJsonTree(const std::string& filename = "", bool states = false, bool writeLinks = false) { return Output::writeJsonTree(*this, filename, states, writeLinks); }
+  std::string writeJsonTree(const std::string& filename = "", bool states = false, bool writeLinks = false) { return infomap::writeJsonTree(*this, m_network, filename, states, writeLinks); }
 
-  std::string writeCsvTree(const std::string& filename = "", bool states = false) { return Output::writeCsvTree(*this, filename, states); }
+  std::string writeCsvTree(const std::string& filename = "", bool states = false) { return infomap::writeCsvTree(*this, m_network, filename, states); }
 
   /**
    * Write tree to a .clu file.
@@ -457,23 +464,14 @@ public:
    * Value -1 will give the module index for the lowest level, i.e. the finest modular structure.
    * @return the filename written to
    */
-  std::string writeClu(const std::string& filename = "", bool states = false, int moduleIndexLevel = 1) { return Output::writeClu(*this, filename, states, moduleIndexLevel); }
-
-  /**
-   * Print per level statistics
-   * @param out The output stream to print the per level statistics to
-   */
-  unsigned int printPerLevelCodelength(std::ostream& out);
-
-  void aggregatePerLevelCodelength(std::vector<PerLevelStat>& perLevelStat, unsigned int level = 0) { aggregatePerLevelCodelength(root(), perLevelStat, level); }
-
-  void aggregatePerLevelCodelength(InfoNode& parent, std::vector<PerLevelStat>& perLevelStat, unsigned int level);
+  std::string writeClu(const std::string& filename = "", bool states = false, int moduleIndexLevel = 1) { return infomap::writeClu(*this, m_network, filename, states, moduleIndexLevel); }
 
+private:
   // ===================================================
   // Debug: *
   // ===================================================
 
-  void printDebug() { return m_optimizer->printDebug(); }
+  void printDebug() const { return m_optimizer->printDebug(); }
 
   // ===================================================
   // Members
@@ -488,7 +486,7 @@ protected:
   std::vector<InfoNode*> m_originalLeafNodes;
 
   Network m_network;
-  std::map<unsigned int, unsigned int> m_initialPartition = {}; // nodeId -> moduleId
+  InitialPartition m_initialPartition = {}; // nodeId -> moduleId
 
   const unsigned int SUPER_LEVEL_ADDITION = 1 << 20;
   bool m_isMain = true;
@@ -516,6 +514,13 @@ protected:
   std::unique_ptr<InfomapOptimizerBase> m_optimizer;
 };
 
+/**
+ * Print per level statistics
+ */
+unsigned int printPerLevelCodelength(const InfoNode& parent, std::ostream& out);
+
+void aggregatePerLevelCodelength(const InfoNode& parent, std::vector<detail::PerLevelStat>& perLevelStat, unsigned int level = 0);
+
 namespace detail {
 
   struct PerLevelStat {
