@@ -1,17 +1,18 @@
-/*
- * InfomapConfig.h
- *
- *  Created on: 2 mar 2015
- *      Author: Daniel
- */
+/*******************************************************************************
+ Infomap software package for multi-level network clustering
+ Copyright (c) 2013, 2014 Daniel Edler, Anton Eriksson, Martin Rosvall
 
-#ifndef MODULES_CLUSTERING_CLUSTERING_INFOMAPCONFIG_H_
-#define MODULES_CLUSTERING_CLUSTERING_INFOMAPCONFIG_H_
+ This file is part of the Infomap software package.
+ See file LICENSE_AGPLv3.txt for full license details.
+ For more information, see <http://www.mapequation.org>
+ ******************************************************************************/
+
+#ifndef INFOMAP_CONFIG_H_
+#define INFOMAP_CONFIG_H_
 
 #include "../io/Config.h"
 #include "../utils/Random.h"
 #include "../utils/Log.h"
-
 #include <string>
 
 namespace infomap {
@@ -21,7 +22,7 @@ class InfomapConfig : public Config {
 public:
   InfomapConfig() = default;
 
-  InfomapConfig(const std::string flags) : InfomapConfig(Config(flags)) {}
+  InfomapConfig(const std::string& flags, bool isCli = false) : InfomapConfig(Config(flags, isCli)) { }
 
   InfomapConfig(const Config& conf) : Config(conf), m_rand(conf.seedToRandomNumberGenerator)
   {
@@ -30,6 +31,11 @@ public:
 
   virtual ~InfomapConfig() = default;
 
+  InfomapConfig(const InfomapConfig&) = default;
+  InfomapConfig& operator=(const InfomapConfig&) = default;
+  InfomapConfig(InfomapConfig&&) noexcept = default;
+  InfomapConfig& operator=(InfomapConfig&&) noexcept = default;
+
 private:
   Infomap& get()
   {
@@ -134,4 +140,4 @@ public:
 
 } // namespace infomap
 
-#endif /* MODULES_CLUSTERING_CLUSTERING_INFOMAPCONFIG_H_ */
+#endif // INFOMAP_CONFIG_H_
