@@ -1,19 +1,24 @@
-/**
- * 
- */
+/*******************************************************************************
+ Infomap software package for multi-level network clustering
+ Copyright (c) 2013, 2014 Daniel Edler, Anton Eriksson, Martin Rosvall
+
+ This file is part of the Infomap software package.
+ See file LICENSE_AGPLv3.txt for full license details.
+ For more information, see <http://www.mapequation.org>
+ ******************************************************************************/
 
 #ifndef INFOMAP_OPTIMIZER_BASE_H_
 #define INFOMAP_OPTIMIZER_BASE_H_
 
 #include "InfomapBase.h"
-#include <vector>
 #include "InfoNode.h"
 #include "FlowData.h"
+#include <vector>
 
 namespace infomap {
 
 class InfomapOptimizerBase {
-  friend class InfomapCore;
+  friend class InfomapBase;
   using FlowDataType = FlowData;
 
 public:
@@ -39,7 +44,7 @@ public:
 
   virtual double getModuleCodelength() const = 0;
 
-  virtual double getMetaCodelength(bool unweighted = false) const { return 0.0; }
+  virtual double getMetaCodelength(bool /*unweighted*/ = false) const { return 0.0; }
 
 protected:
   virtual unsigned int numActiveModules() const = 0;
@@ -86,4 +91,4 @@ protected:
 
 } /* namespace infomap */
 
-#endif /* INFOMAP_OPTIMIZER_BASE_H_ */
+#endif // INFOMAP_OPTIMIZER_BASE_H_
