@@ -1,65 +1,44 @@
-/*
- * MetaMapEquation.h
- */
+/*******************************************************************************
+ Infomap software package for multi-level network clustering
+ Copyright (c) 2013, 2014 Daniel Edler, Anton Eriksson, Martin Rosvall
 
-#ifndef _METAMAPEQUATION_H_
-#define _METAMAPEQUATION_H_
+ This file is part of the Infomap software package.
+ See file LICENSE_AGPLv3.txt for full license details.
+ For more information, see <http://www.mapequation.org>
+ ******************************************************************************/
 
+#ifndef META_MAPEQUATION_H_
+#define META_MAPEQUATION_H_
+
+#include "MapEquation.h"
+#include "FlowData.h"
+#include "../utils/Log.h"
+#include "../utils/MetaCollection.h"
 #include <vector>
 #include <set>
 #include <map>
 #include <utility>
-#include "MapEquation.h"
-#include "FlowData.h"
-// #include "InfoNode.h"
-#include "../utils/Log.h"
-#include "../utils/MetaCollection.h"
 
 namespace infomap {
 
 class InfoNode;
 
-class MetaMapEquation : protected MapEquation {
-  using Base = MapEquation;
+class MetaMapEquation : private MapEquation<> {
+  using Base = MapEquation<>;
 
 public:
   using FlowDataType = FlowData;
   using DeltaFlowDataType = DeltaFlow;
 
-  MetaMapEquation() = default;
-
-  MetaMapEquation(const MetaMapEquation& other)
-      : MapEquation(other),
-        m_moduleToMetaCollection(other.m_moduleToMetaCollection),
-        numMetaDataDimensions(other.numMetaDataDimensions),
-        metaDataRate(other.metaDataRate),
-        weightByFlow(other.weightByFlow),
-        metaCodelength(other.metaCodelength) {}
-
-  MetaMapEquation& operator=(const MetaMapEquation& other)
-  {
-    Base::operator=(other);
-    m_moduleToMetaCollection = other.m_moduleToMetaCollection;
-    numMetaDataDimensions = other.numMetaDataDimensions;
-    metaDataRate = other.metaDataRate;
-    weightByFlow = other.weightByFlow;
-    metaCodelength = other.metaCodelength;
-    return *this;
-  }
-
-  virtual ~MetaMapEquation() = default;
-
   // ===================================================
   // Getters
   // ===================================================
 
   using Base::getIndexCodelength;
 
-  // double getModuleCodelength() const { return moduleCodelength + metaCodelength; };
-  double getModuleCodelength() const;
+  double getModuleCodelength() const override;
 
-  // double getCodelength() const { return codelength + metaCodelength; };
-  double getCodelength() const;
+  double getCodelength() const override;
 
   double getMetaCodelength(bool unweighted = false) const
   {
@@ -70,63 +49,62 @@ public:
   // IO
   // ===================================================
 
-  // using Base::print;
-  std::ostream& print(std::ostream& out) const;
+  std::ostream& print(std::ostream& out) const override;
   friend std::ostream& operator<<(std::ostream&, const MetaMapEquation&);
 
   // ===================================================
   // Init
   // ===================================================
 
-  void init(const Config& config);
+  void init(const Config& config) override;
 
-  void initTree(InfoNode& root);
+  void initTree(InfoNode& root) override;
 
-  void initNetwork(InfoNode& root);
+  void initNetwork(InfoNode& root) override;
 
-  void initSuperNetwork(InfoNode& root);
+  using Base::initSuperNetwork;
 
-  void initSubNetwork(InfoNode& root);
+  using Base::initSubNetwork;
 
-  void initPartition(std::vector<InfoNode*>& nodes);
+  void initPartition(std::vector<InfoNode*>& nodes) override;
 
   // ===================================================
   // Codelength
   // ===================================================
 
-  double calcCodelength(const InfoNode& parent) const;
+  double calcCodelength(const InfoNode& parent) const override;
 
-  void addMemoryContributions(InfoNode& current, DeltaFlowDataType& oldModuleDelta, VectorMap<DeltaFlowDataType>& moduleDeltaFlow) {}
+  using Base::addMemoryContributions;
 
   double getDeltaCodelengthOnMovingNode(InfoNode& current,
-                                        DeltaFlowDataType& oldModuleDelta,
-                                        DeltaFlowDataType& newModuleDelta,
-                                        std::vector<FlowDataType>& moduleFlowData,
-                                        std::vector<unsigned int>& moduleMembers);
+                                        DeltaFlow& oldModuleDelta,
+                                        DeltaFlow& newModuleDelta,
+                                        std::vector<FlowData>& moduleFlowData,
+                                        std::vector<unsigned int>& moduleMembers) override;
 
   // ===================================================
   // Consolidation
   // ===================================================
 
   void updateCodelengthOnMovingNode(InfoNode& current,
-                                    DeltaFlowDataType& oldModuleDelta,
-                                    DeltaFlowDataType& newModuleDelta,
-                                    std::vector<FlowDataType>& moduleFlowData,
-                                    std::vector<unsigned int>& moduleMembers);
+                                    DeltaFlow& oldModuleDelta,
+                                    DeltaFlow& newModuleDelta,
+                                    std::vector<FlowData>& moduleFlowData,
+                                    std::vector<unsigned int>& moduleMembers) override;
 
-  void consolidateModules(std::vector<InfoNode*>& modules);
+  void consolidateModules(std::vector<InfoNode*>& modules) override;
 
   // ===================================================
   // Debug
   // ===================================================
 
-  void printDebug();
+  void printDebug() const override;
 
-protected:
+private:
   // ===================================================
-  // Protected member functions
+  // Private member functions
   // ===================================================
-  double calcCodelengthOnModuleOfLeafNodes(const InfoNode& parent) const;
+  double calcCodelengthOnModuleOfLeafNodes(const InfoNode& parent) const override;
 
   // ===================================================
   // Init
@@ -140,7 +118,7 @@ protected:
   // Codelength
   // ===================================================
 
-  void calculateCodelength(std::vector<InfoNode*>& nodes);
+  void calculateCodelength(std::vector<InfoNode*>& nodes) override;
 
   using Base::calculateCodelengthTerms;
 
@@ -161,9 +139,9 @@ public:
   using Base::indexCodelength;
   using Base::moduleCodelength;
 
-protected:
+private:
   // ===================================================
-  // Protected member functions
+  // Private member functions
   // ===================================================
 
   /**
@@ -174,7 +152,7 @@ protected:
   double getCurrentModuleMetaCodelength(unsigned int module, InfoNode& current, int addRemoveOrNothing);
 
   // ===================================================
-  // Protected member variables
+  // Private member variables
   // ===================================================
 
   using Base::enter_log_enter;
@@ -200,7 +178,6 @@ protected:
   double m_unweightedNodeFlow = 0.0;
 };
 
-
 } // namespace infomap
 
-#endif /* _METAMAPEQUATION_H_ */
+#endif // META_MAPEQUATION_H_
