@@ -1,12 +1,14 @@
-/*
- * Log.h
- *
- *  Created on: 26 feb 2015
- *      Author: Daniel
- */
+/*******************************************************************************
+ Infomap software package for multi-level network clustering
+ Copyright (c) 2013, 2014 Daniel Edler, Anton Eriksson, Martin Rosvall
 
-#ifndef _LOG_H_
-#define _LOG_H_
+ This file is part of the Infomap software package.
+ See file LICENSE_AGPLv3.txt for full license details.
+ For more information, see <http://www.mapequation.org>
+ ******************************************************************************/
+
+#ifndef LOG_H_
+#define LOG_H_
 
 #include <iostream>
 #include <limits>
@@ -26,7 +28,7 @@ public:
    * and maxLevel is above or equal Log::verboseLevel()
    */
   explicit Log(unsigned int level = 0, unsigned int maxLevel = std::numeric_limits<int>::max())
-      : m_level(level), m_maxLevel(maxLevel), m_visible(isVisible(m_level, m_maxLevel)) {}
+      : m_level(level), m_maxLevel(maxLevel), m_visible(isVisible(m_level, m_maxLevel)) { }
 
   bool isVisible() const { return isVisible(m_level, m_maxLevel); }
 
@@ -53,7 +55,7 @@ public:
   {
     setVerboseLevel(verboseLevel);
     setSilent(silent);
-    Log() << std::setprecision(numberPrecision);
+    Log() << std::setprecision(static_cast<int>(numberPrecision));
   }
 
   static bool isVisible(unsigned int level, unsigned int maxLevel)
@@ -63,8 +65,6 @@ public:
 
   static void setVerboseLevel(unsigned int level) { s_verboseLevel = level; }
 
-  static unsigned int verboseLevel() { return s_verboseLevel; }
-
   static void setSilent(bool silent) { s_silent = silent; }
 
   static bool isSilent() { return s_silent; }
@@ -87,7 +87,7 @@ private:
 };
 
 struct hideIf {
-  explicit hideIf(bool value) : hide(value) {}
+  explicit hideIf(bool value) : hide(value) { }
 
   friend Log& operator<<(Log& out, const hideIf& manip)
   {
@@ -100,4 +100,4 @@ struct hideIf {
 
 } // namespace infomap
 
-#endif /* _LOG_H_ */
+#endif // LOG_H_
