@@ -17,6 +17,7 @@
 #include "InfomapOptimizer.h"
 #include "../io/SafeFile.h"
 #include "../utils/FileURI.h"
+#include "../utils/FlowCalculator.h"
 
 #include <stdexcept>
 #include <string>
@@ -35,6 +36,33 @@
 
 namespace infomap {
 
+std::map<unsigned int, std::vector<unsigned int>> InfomapBase::getMultilevelModules(bool states)
+{
+  unsigned int maxDepth = maxTreeDepth();
+  unsigned int numModuleLevels = maxDepth - 1;
+  std::map<unsigned int, std::vector<unsigned int>> modules;
+  if (maxDepth < 2) return modules;
+  for (unsigned int level = 1; level <= numModuleLevels; ++level) {
+    if (haveMemory() && !states) {
+      for (auto it(iterTreePhysical(static_cast<int>(level))); !it.isEnd(); ++it) {
+        auto& node = *it;
+        if (node.isLeaf()) {
+          modules[node.physicalId].push_back(it.moduleId());
+        }
+      }
+    } else {
+      for (auto it(iterTree(static_cast<int>(level))); !it.isEnd(); ++it) {
+        auto& node = *it;
+        if (node.isLeaf()) {
+          auto nodeId = states ? node.stateId : node.physicalId;
+          modules[nodeId].push_back(it.moduleId());
+        }
+      }
+    }
+  }
+  return modules;
+}
+
 // ===================================================
 // IO
 // ===================================================
@@ -195,7 +223,7 @@ void InfomapBase::run(Network& network)
   }
   network.setConfig(*this);
 
-  network.calculateFlow();
+  calculateFlow(network, *this);
 
   if (printFlowNetwork) {
     std::string filename;
@@ -260,7 +288,7 @@ void InfomapBase::run(Network& network)
     if (!noInfomap)
       runPartition();
     else
-      m_hierarchicalCodelength = calcCodelengthOnTree(true);
+      m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
 
     if (haveHardPartition())
       restoreHardPartition();
@@ -268,13 +296,18 @@ void InfomapBase::run(Network& network)
     if (isMainInfomap()) {
       Log() << "\n=> Trial " << (i + 1) << "/" << numTrials << " finished in " << timer.getElapsedTimeInSec() << "s with codelength " << m_hierarchicalCodelength << "\n";
       m_codelengths.push_back(m_hierarchicalCodelength);
+
+      if (printAllTrials && numTrials > 1) {
+        writeResult(static_cast<int>(i + 1));
+      }
+
       if (m_hierarchicalCodelength < bestHierarchicalCodelength - 1e-10) {
         bestSolutionStatistics.clear();
         bestSolutionStatistics.str("");
-        bestNumLevels = printPerLevelCodelength(bestSolutionStatistics);
+        bestNumLevels = printPerLevelCodelength(root(), bestSolutionStatistics);
         bestHierarchicalCodelength = m_hierarchicalCodelength;
         bestTrialIndex = i;
-        sortTreeOnFlow();
+        root().sortChildrenOnFlow();
         writeResult();
         if (numTrials > 1) {
           bestTree.clear();
@@ -496,7 +529,7 @@ InfomapBase& InfomapBase::initTree(const NodePaths& tree)
   aggregateFlowValuesFromLeafToRoot();
   initTree();
   initNetwork();
-  calculateNumNonTrivialTopModules();
+  m_numNonTrivialTopModules = calculateNumNonTrivialTopModules();
   // Init partition to calculate indexCodelength and moduleCodelength
   if (haveModules())
     setActiveNetworkFromChildrenOfRoot();
@@ -504,7 +537,7 @@ InfomapBase& InfomapBase::initTree(const NodePaths& tree)
     setActiveNetworkFromLeafs();
   initPartition();
 
-  m_hierarchicalCodelength = calcCodelengthOnTree(true);
+  m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
   Log() << "\n -> Initiated to codelength " << m_hierarchicalCodelength << " in " << maxDepth << " levels with " << numTopModules() << " top modules.\n";
   Log() << std::setprecision(6);
   return *this;
@@ -782,6 +815,7 @@ void InfomapBase::init()
   Log() << "done!\n -> One-level codelength: " << m_oneLevelCodelength << '\n';
 
   Log() << "Calculating entropy rate... " << std::flush;
+
   double entropyRate = calcEntropyRate();
   Log() << "done!\n  -> Entropy rate: " << io::toPrecision(entropyRate) << '\n';
 }
@@ -820,7 +854,7 @@ void InfomapBase::hierarchicalPartition()
     if (fastHierarchicalSolution == 0) {
       Log(1) << "Removing sub modules...\n";
       removeSubModules(true);
-      m_hierarchicalCodelength = calcCodelengthOnTree(true);
+      m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
     }
 
     else if (fastHierarchicalSolution == 1) {
@@ -910,7 +944,7 @@ void InfomapBase::hierarchicalPartition()
 
   if (onlySuperModules) {
     removeSubModules(true);
-    m_hierarchicalCodelength = calcCodelengthOnTree(true);
+    m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
     return;
   }
 
@@ -921,7 +955,7 @@ void InfomapBase::hierarchicalPartition()
 
   if (fastHierarchicalSolution == 0) {
     removeSubModules(true);
-    m_hierarchicalCodelength = calcCodelengthOnTree(true);
+    m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
   }
 
   recursivePartition();
@@ -1013,11 +1047,11 @@ void InfomapBase::partition()
     }
 
     // Calculate individual module codelengths and store on the modules
-    calcCodelengthOnTree(false);
+    calcCodelengthOnTree(root(), false);
     m_root.codelength = getIndexCodelength();
     m_hierarchicalCodelength = getCodelength();
   }
-  m_hierarchicalCodelength = calcCodelengthOnTree(true);
+  m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
 }
 
 void InfomapBase::restoreHardPartition()
@@ -1064,7 +1098,7 @@ void InfomapBase::initEnterExitFlow()
   if (!isUndirectedClustering()) {
     for (auto* n : m_leafNodes) {
       auto& node = *n;
-      node.data.teleportFlow = teleportationProbability * node.data.flow;
+      node.data.teleportFlow = alpha * node.data.flow;
       node.data.teleportSourceFlow = node.data.flow;
       if (node.isDangling()) {
         node.data.teleportFlow = node.data.flow;
@@ -1089,7 +1123,7 @@ void InfomapBase::initEnterExitFlow()
   } else {
     for (auto* n : m_leafNodes) {
       auto& node = *n;
-      node.data.teleportFlow = teleportationProbability * node.data.flow;
+      node.data.teleportFlow = alpha * node.data.flow;
       node.data.teleportSourceFlow = node.data.flow;
       if (node.isDangling()) {
         node.data.teleportFlow = node.data.flow;
@@ -1203,11 +1237,11 @@ void InfomapBase::aggregateFlowValuesFromLeafToRoot()
   }
 }
 
-double InfomapBase::calcCodelengthOnTree(bool includeRoot)
+double InfomapBase::calcCodelengthOnTree(InfoNode& root, bool includeRoot) const
 {
   double totalCodelength = 0.0;
   // Calculate enter/exit flow (assuming 0 by default)
-  for (auto& node : m_root.infomapTree()) {
+  for (auto& node : root.infomapTree()) {
     if (node.isLeaf() || (!includeRoot && node.isRoot()))
       continue;
     node.codelength = calcCodelength(node);
@@ -1273,7 +1307,7 @@ void InfomapBase::findTopModulesRepeatedly(unsigned int maxLevels)
   }
   m_aggregationLevel = 0;
 
-  calculateNumNonTrivialTopModules();
+  m_numNonTrivialTopModules = calculateNumNonTrivialTopModules();
 
   Log(1, 2) << (m_isCoarseTune ? "modules" : "nodes") << "*loops from codelength " << initialCodelength << " to codelength " << *this << " in " << numTopModules() << " modules. (" << m_numNonTrivialTopModules << " non-trivial modules)\n";
 }
@@ -1399,18 +1433,19 @@ unsigned int InfomapBase::coarseTune()
   return numEffectiveLoops;
 }
 
-void InfomapBase::calculateNumNonTrivialTopModules()
+unsigned int InfomapBase::calculateNumNonTrivialTopModules() const
 {
-  m_numNonTrivialTopModules = 0;
   if (root().childDegree() == 1)
-    return;
+    return 0;
+  unsigned int numNonTrivialTopModules = 0;
   for (auto& module : m_root) {
     if (module.childDegree() > 1)
-      ++m_numNonTrivialTopModules;
+      ++numNonTrivialTopModules;
   }
+  return numNonTrivialTopModules;
 }
 
-unsigned int InfomapBase::calculateMaxDepth()
+unsigned int InfomapBase::calculateMaxDepth() const
 {
   unsigned int maxDepth = 0;
   for (auto it(m_root.begin_tree()); !it.isEnd(); ++it) {
@@ -1480,7 +1515,7 @@ unsigned int InfomapBase::findHierarchicalSuperModulesFast(unsigned int superLev
 
     ++numLevelsCreated;
 
-    calculateNumNonTrivialTopModules();
+    m_numNonTrivialTopModules = calculateNumNonTrivialTopModules();
 
   } while (m_numNonTrivialTopModules > 1 && numLevelsCreated != superLevelLimit);
 
@@ -1491,7 +1526,7 @@ unsigned int InfomapBase::findHierarchicalSuperModulesFast(unsigned int superLev
   Log(1) << "Finding super modules done! Added " << numLevelsCreated << " levels with hierarchical codelength " << io::toPrecision(hierarchicalCodelength) << " in " << numTopModules() << " top modules.\n";
 
   // Calculate and store hierarchical codelengths
-  m_hierarchicalCodelength = calcCodelengthOnTree(true);
+  m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
 
   return numLevelsCreated;
 }
@@ -1574,7 +1609,7 @@ unsigned int InfomapBase::findHierarchicalSuperModules(unsigned int superLevelLi
 
     ++numLevelsCreated;
 
-    calculateNumNonTrivialTopModules();
+    m_numNonTrivialTopModules = calculateNumNonTrivialTopModules();
 
   } while (m_numNonTrivialTopModules > 1 && numLevelsCreated != superLevelLimit);
 
@@ -1585,7 +1620,7 @@ unsigned int InfomapBase::findHierarchicalSuperModules(unsigned int superLevelLi
   Log(1) << "Finding super modules done! Added " << numLevelsCreated << " levels with hierarchical codelength " << io::toPrecision(hierarchicalCodelength) << " in " << numTopModules() << " top modules.\n";
 
   // Calculate and store hierarchical codelengths
-  m_hierarchicalCodelength = calcCodelengthOnTree(true);
+  m_hierarchicalCodelength = calcCodelengthOnTree(root(), true);
 
   return numLevelsCreated;
 }
@@ -1642,7 +1677,7 @@ unsigned int InfomapBase::removeSubModules(bool recalculateCodelengthOnTree)
   if (numLevelsDeleted > 0) {
     Log(2) << "Removed " << numLevelsDeleted << " levels of sub-modules.\n";
     if (recalculateCodelengthOnTree)
-      calcCodelengthOnTree(false);
+      calcCodelengthOnTree(root(), false);
   }
   return numLevelsDeleted;
 }
@@ -1775,7 +1810,7 @@ void InfomapBase::queueLeafModules(PartitionQueue& partitionQueue)
   partitionQueue.level = maxDepth;
 }
 
-bool InfomapBase::processPartitionQueue(PartitionQueue& queue, PartitionQueue& nextLevelQueue)
+bool InfomapBase::processPartitionQueue(PartitionQueue& queue, PartitionQueue& nextLevelQueue) const
 {
   PartitionQueue::size_t numModules = queue.size();
   std::vector<double> indexCodelengths(numModules, 0.0);
@@ -1868,13 +1903,22 @@ bool InfomapBase::processPartitionQueue(PartitionQueue& queue, PartitionQueue& n
 // Write output
 // ===================================================
 
-void InfomapBase::writeResult()
+void InfomapBase::writeResult(int trial)
 {
   if (noFileOutput)
     return;
 
+  io::Str s;
+  s << outDirectory + outName;
+
+  if (printAllTrials && trial != -1 && numTrials > 1) {
+    s << "_trial_" << trial;
+  }
+
+  std::string basename = s;
+
   if (printTree) {
-    std::string filename = outDirectory + outName + ".tree";
+    std::string filename = basename + ".tree";
 
     if (!printStates()) {
       Log() << "Write tree to " << filename << "... ";
@@ -1885,7 +1929,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical tree to " << filename << "... ";
       writeTree(filename);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.tree";
+      std::string filenameStates = basename + "_states.tree";
       Log() << "Write state tree to " << filenameStates << "... ";
       writeTree(filenameStates, true);
       Log() << "done!\n";
@@ -1893,7 +1937,7 @@ void InfomapBase::writeResult()
   }
 
   if (printFlowTree) {
-    std::string filename = outDirectory + outName + ".ftree";
+    std::string filename = basename + ".ftree";
 
     if (!printStates()) {
       Log() << "Write flow tree to " << filename << "... ";
@@ -1904,7 +1948,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical flow tree to " << filename << "... ";
       writeFlowTree(filename, false);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.ftree";
+      std::string filenameStates = basename + "_states.ftree";
       Log() << "Write state flow tree to " << filenameStates << "... ";
       writeFlowTree(filenameStates, true);
       Log() << "done!\n";
@@ -1912,7 +1956,7 @@ void InfomapBase::writeResult()
   }
 
   if (printNewick) {
-    std::string filename = outDirectory + outName + ".nwk";
+    std::string filename = basename + ".nwk";
 
     if (!printStates()) {
       Log() << "Write Newick tree to " << filename << "... ";
@@ -1923,7 +1967,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical Newick tree to " << filename << "... ";
       writeNewickTree(filename, false);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.nwk";
+      std::string filenameStates = basename + "_states.nwk";
       Log() << "Write state Newick tree to " << filenameStates << "... ";
       writeNewickTree(filenameStates, true);
       Log() << "done!\n";
@@ -1931,7 +1975,7 @@ void InfomapBase::writeResult()
   }
 
   if (printJson) {
-    std::string filename = outDirectory + outName + ".json";
+    std::string filename = basename + ".json";
     const bool writeLinks = false;
 
     if (!printStates()) {
@@ -1943,7 +1987,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical JSON tree to " << filename << "... ";
       writeJsonTree(filename, false, writeLinks);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.json";
+      std::string filenameStates = basename + "_states.json";
       Log() << "Write state JSON tree to " << filenameStates << "... ";
       writeJsonTree(filenameStates, true, writeLinks);
       Log() << "done!\n";
@@ -1951,7 +1995,7 @@ void InfomapBase::writeResult()
   }
 
   if (printCsv) {
-    std::string filename = outDirectory + outName + ".csv";
+    std::string filename = basename + ".csv";
 
     if (!printStates()) {
       Log() << "Write CSV tree to " << filename << "... ";
@@ -1962,7 +2006,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical CSV tree to " << filename << "... ";
       writeCsvTree(filename, false);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.csv";
+      std::string filenameStates = basename + "_states.csv";
       Log() << "Write state CSV tree to " << filenameStates << "... ";
       writeCsvTree(filenameStates, true);
       Log() << "done!\n";
@@ -1970,7 +2014,7 @@ void InfomapBase::writeResult()
   }
 
   if (printClu) {
-    std::string filename = outDirectory + outName + ".clu";
+    std::string filename = basename + ".clu";
     if (!printStates()) {
       Log() << "Write node modules to " << filename << "... ";
       writeClu(filename, false, cluLevel);
@@ -1980,7 +2024,7 @@ void InfomapBase::writeResult()
       Log() << "Write physical node modules to " << filename << "... ";
       writeClu(filename, false, cluLevel);
       Log() << "done!\n";
-      std::string filenameStates = outDirectory + outName + "_states.clu";
+      std::string filenameStates = basename + "_states.clu";
       Log() << "Write state node modules to " << filenameStates << "... ";
       writeClu(filenameStates, true, cluLevel);
       Log() << "done!\n";
@@ -1988,10 +2032,10 @@ void InfomapBase::writeResult()
   }
 }
 
-unsigned int InfomapBase::printPerLevelCodelength(std::ostream& out)
+unsigned int printPerLevelCodelength(const InfoNode& parent, std::ostream& out)
 {
-  std::vector<PerLevelStat> perLevelStats;
-  aggregatePerLevelCodelength(perLevelStats);
+  std::vector<detail::PerLevelStat> perLevelStats;
+  aggregatePerLevelCodelength(parent, perLevelStats);
 
   unsigned int numLevels = perLevelStats.size();
 
@@ -2069,7 +2113,7 @@ unsigned int InfomapBase::printPerLevelCodelength(std::ostream& out)
   return numLevels;
 }
 
-void InfomapBase::aggregatePerLevelCodelength(InfoNode& parent, std::vector<PerLevelStat>& perLevelStat, unsigned int level)
+void aggregatePerLevelCodelength(const InfoNode& parent, std::vector<detail::PerLevelStat>& perLevelStat, unsigned int level)
 {
   if (perLevelStat.size() < level + 1)
     perLevelStat.resize(level + 1);
@@ -2085,7 +2129,7 @@ void InfomapBase::aggregatePerLevelCodelength(InfoNode& parent, std::vector<PerL
 
   for (auto& module : parent) {
     if (module.getInfomapRoot() != nullptr)
-      module.getInfomap().aggregatePerLevelCodelength(perLevelStat, level + 1);
+      aggregatePerLevelCodelength(*module.getInfomapRoot(), perLevelStat, level + 1);
     else
       aggregatePerLevelCodelength(module, perLevelStat, level + 1);
   }
